@@ -7,6 +7,7 @@ use crate::ipc::context::{Context, PooledContext, ReplyListeners};
 use crate::ipc::server::IPCServer;
 use crate::namespaces::builder::NamespaceBuilder;
 use crate::namespaces::namespace::Namespace;
+use crate::protocol::AsyncStreamProtocolListener;
 use std::collections::HashMap;
 use std::future::Future;
 use std::pin::Pin;
@@ -16,8 +17,10 @@ use typemap_rev::{TypeMap, TypeMapKey};
 
 /// A builder for the IPC server or client.
 /// ```no_run
-///use typemap_rev::TypeMapKey;
+/// use std::net::ToSocketAddrs;
+/// use typemap_rev::TypeMapKey;
 /// use rmp_ipc::IPCBuilder;
+/// use tokio::net::TcpListener;
 ///
 /// struct CustomKey;
 ///
@@ -26,8 +29,8 @@ use typemap_rev::{TypeMap, TypeMapKey};
 /// }
 ///
 ///# async fn a() {
-/// IPCBuilder::new()
-///     .address("127.0.0.1:2020")
+/// IPCBuilder::<TcpListener>::new()
+///     .address("127.0.0.1:2020".to_socket_addrs().unwrap().next().unwrap())
 ///    // register callback
 ///     .on("ping", |_ctx, _event| Box::pin(async move {
 ///         println!("Received ping event.");
@@ -46,14 +49,17 @@ use typemap_rev::{TypeMap, TypeMapKey};
 ///     .build_server().await.unwrap();
 ///# }
 /// ```
-pub struct IPCBuilder {
-    handler: EventHandler,
-    address: Option<String>,
-    namespaces: HashMap<String, Namespace>,
+pub struct IPCBuilder<L: AsyncStreamProtocolListener> {
+    handler: EventHandler<L::Stream>,
+    address: Option<L::AddressType>,
+    namespaces: HashMap<String, Namespace<L::Stream>>,
     data: TypeMap,
 }
 
-impl IPCBuilder {
+impl<L> IPCBuilder<L>
+where
+    L: AsyncStreamProtocolListener,
+{
     pub fn new() -> Self {
         let mut handler = EventHandler::new();
         handler.on(ERROR_EVENT_NAME, |_, event| {
@@ -84,7 +90,7 @@ impl IPCBuilder {
     pub fn on<F: 'static>(mut self, event: &str, callback: F) -> Self
     where
         F: for<'a> Fn(
-                &'a Context,
+                &'a Context<L::Stream>,
                 Event,
             ) -> Pin<Box<(dyn Future<Output = Result<()>> + Send + 'a)>>
             + Send
@@ -96,19 +102,19 @@ impl IPCBuilder {
     }
 
     /// Adds the address to connect to
-    pub fn address<S: ToString>(mut self, address: S) -> Self {
-        self.address = Some(address.to_string());
+    pub fn address(mut self, address: L::AddressType) -> Self {
+        self.address = Some(address);
 
         self
     }
 
     /// Adds a namespace
-    pub fn namespace<S: ToString>(self, name: S) -> NamespaceBuilder {
+    pub fn namespace<S: ToString>(self, name: S) -> NamespaceBuilder<L> {
         NamespaceBuilder::new(self, name.to_string())
     }
 
     /// Adds a namespace to the ipc server
-    pub fn add_namespace(mut self, namespace: Namespace) -> Self {
+    pub fn add_namespace(mut self, namespace: Namespace<L::Stream>) -> Self {
         self.namespaces
             .insert(namespace.name().to_owned(), namespace);
 
@@ -119,19 +125,19 @@ impl IPCBuilder {
     #[tracing::instrument(skip(self))]
     pub async fn build_server(self) -> Result<()> {
         self.validate()?;
-        let server = IPCServer {
+        let server = IPCServer::<L> {
             namespaces: self.namespaces,
             handler: self.handler,
             data: self.data,
         };
-        server.start(&self.address.unwrap()).await?;
+        server.start(self.address.unwrap()).await?;
 
         Ok(())
     }
 
     /// Builds an ipc client
     #[tracing::instrument(skip(self))]
-    pub async fn build_client(self) -> Result<Context> {
+    pub async fn build_client(self) -> Result<Context<L::Stream>> {
         self.validate()?;
         let data = Arc::new(RwLock::new(self.data));
         let reply_listeners = ReplyListeners::default();
@@ -142,7 +148,7 @@ impl IPCBuilder {
             reply_listeners,
         };
 
-        let ctx = client.connect(&self.address.unwrap()).await?;
+        let ctx = client.connect(self.address.unwrap()).await?;
 
         Ok(ctx)
     }
@@ -152,7 +158,7 @@ impl IPCBuilder {
     /// return a [crate::context::PooledContext] that allows one to [crate::context::PooledContext::acquire] a single context
     /// to emit events.
     #[tracing::instrument(skip(self))]
-    pub async fn build_pooled_client(self, pool_size: usize) -> Result<PooledContext> {
+    pub async fn build_pooled_client(self, pool_size: usize) -> Result<PooledContext<L::Stream>> {
         if pool_size == 0 {
             Error::BuildError("Pool size must be greater than 0".to_string());
         }
@@ -170,7 +176,7 @@ impl IPCBuilder {
                 reply_listeners: Arc::clone(&reply_listeners),
             };
 
-            let ctx = client.connect(&address).await?;
+            let ctx = client.connect(address.clone()).await?;
             contexts.push(ctx);
         }
 
