@@ -3,9 +3,10 @@
 //! Client Example:
 //! ```no_run
 //! use rmp_ipc::prelude::*;
+//! use tokio::net::TcpListener;
 //!
 //! /// Callback ping function
-//! async fn handle_ping(ctx: &Context, event: Event) -> IPCResult<()> {
+//! async fn handle_ping<S: AsyncProtocolStream>(ctx: &Context<S>, event: Event) -> IPCResult<()> {
 //!     println!("Received ping event.");
 //!     ctx.emitter.emit_response(event.id(), "pong", ()).await?;
 //!
@@ -15,7 +16,7 @@
 //! pub struct MyNamespace;
 //!
 //! impl MyNamespace {
-//!     async fn ping(_ctx: &Context, _event: Event) -> IPCResult<()> {
+//!     async fn ping<S: AsyncProtocolStream>(_ctx: &Context<S>, _event: Event) -> IPCResult<()> {
 //!         println!("My namespace received a ping");
 //!         Ok(())
 //!     }
@@ -24,7 +25,7 @@
 //! impl NamespaceProvider for MyNamespace {
 //!     fn name() -> &'static str {"my_namespace"}
 //!
-//!     fn register(handler: &mut EventHandler) {
+//!     fn register<S: AsyncProtocolStream>(handler: &mut EventHandler<S>) {
 //!         events!(handler,
 //!             "ping" => Self::ping,
 //!             "ping2" => Self::ping
@@ -35,8 +36,9 @@
 //! #[tokio::main]
 //! async fn main() {
 //!     // create the client
-//!     let ctx = IPCBuilder::new()
-//!         .address("127.0.0.1:2020")
+//!     use std::net::ToSocketAddrs;
+//! let ctx = IPCBuilder::<TcpListener>::new()
+//!         .address("127.0.0.1:2020".to_socket_addrs().unwrap().next().unwrap())
 //!         // register callback
 //!         .on("ping", callback!(handle_ping))
 //!         .namespace("mainspace-client")
@@ -58,9 +60,11 @@
 //!
 //! Server Example:
 //! ```no_run
+//! use std::net::ToSocketAddrs;
 //! use typemap_rev::TypeMapKey;
 //! use rmp_ipc::IPCBuilder;
 //! use rmp_ipc::callback;
+//! use tokio::net::TcpListener;
 //!
 //! struct MyKey;
 //!
@@ -70,8 +74,8 @@
 //!
 //! // create the server
 //!# async fn a() {
-//! IPCBuilder::new()
-//!     .address("127.0.0.1:2020")
+//! IPCBuilder::<TcpListener>::new()
+//!     .address("127.0.0.1:2020".to_socket_addrs().unwrap().next().unwrap())
 //!     // register callback
 //!     .on("ping", callback!(ctx, event, async move {
 //!         println!("Received ping event.");
@@ -105,6 +109,7 @@ mod events;
 pub mod ipc;
 mod macros;
 mod namespaces;
+pub mod protocol;
 
 pub use events::error_event;
 pub use events::event;
@@ -130,5 +135,6 @@ pub mod prelude {
     pub use crate::namespaces::builder::NamespaceBuilder;
     pub use crate::namespaces::provider_trait::*;
     pub use crate::payload::*;
+    pub use crate::protocol::*;
     pub use crate::*;
 }
